/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package net.java.slee.resource.diameter.s6a.events.avp;

import java.io.Serializable;
import net.java.slee.resource.diameter.base.events.avp.Enumerated;

/**
 * Java class representing the Pre-emption-Vulnerability enumerated type.
 * From the Diameter Gx Reference Point Protocol Details (3GPP TS 29.212 V9.6.0) specification:
 * 
 * <pre>
 * 5.3.47  Pre-emption-Vulnerability AVP
 * 
 * The Pre-emption Vulnerability AVP (AVP code 1048) is of type Enumerated. The AVP defines whether
 * a service data flow can lose the resources assigned to it in order to admit a service data flow
 * with higher priority level.
 * 
 * The following values are defined:
 * 
 *   PRE-EMPTION_VULNERABILITY_ENABLED (0)
 * This value indicates that the resources assigned to the service data flow can be pre-empted and
 * allocated to a service data flow with a higher priority level. This is the default value
 * applicable if this AVP is not supplied.
 * 
 *   PRE-EMPTION_VULNERABILITY_DISABLED (1)
 * This value indicates that the resources assigned to the service data flow shall not be
 * pre-empted and allocated to a service data flow with a higher priority level.
 * </pre>
 * 
 * @author <a href="mailto:brainslog@gmail.com"> Alexandre Mendonca </a>
 * @author <a href="mailto:baranowb@gmail.com"> Bartosz Baranowski </a>
 * @author <a href="mailto:richard.good@smilecoms.com"> Richard Good </a>
 * @author <a href="mailto:paul.carter-brown@smilecoms.com"> Paul Carter-Brown </a>
 */
public class PreEmptionVulnerability implements Enumerated, Serializable {

  private static final long serialVersionUID = 1L;

  public static final int _PRE_EMPTION_VULNERABILITY_ENABLED = 0;
  public static final int _PRE_EMPTION_VULNERABILITY_DISABLED = 1;

  public static final PreEmptionVulnerability PRE_EMPTION_VULNERABILITY_ENABLED = new PreEmptionVulnerability(_PRE_EMPTION_VULNERABILITY_ENABLED);
  public static final PreEmptionVulnerability PRE_EMPTION_VULNERABILITY_DISABLED = new PreEmptionVulnerability(_PRE_EMPTION_VULNERABILITY_DISABLED);

  private int value = -1;

  private PreEmptionVulnerability(int value) {
    this.value = value;
  }

  public static PreEmptionVulnerability fromInt(int type) {
    switch (type) {
      case _PRE_EMPTION_VULNERABILITY_ENABLED:
        return PRE_EMPTION_VULNERABILITY_ENABLED;
      case _PRE_EMPTION_VULNERABILITY_DISABLED:
        return PRE_EMPTION_VULNERABILITY_DISABLED;
      default:
        throw new IllegalArgumentException("Invalid value: " + type);
    }
  }

  public int getValue() {
    return value;
  }

  @Override
  public String toString() {
    switch (value) {
      case _PRE_EMPTION_VULNERABILITY_ENABLED:
        return "PRE-EMPTION_VULNERABILITY_ENABLED";
      case _PRE_EMPTION_VULNERABILITY_DISABLED:
        return "PRE-EMPTION_VULNERABILITY_DISABLED";
      default:
        return "<Invalid Value>";
    }
  }
}